import { CollectionData, CollectionRequest } from './interfaces/collection.interface';
import { BehaviorSubject, Observable } from 'rxjs';
import { CollectionProvider } from './providers/collection.provider';
import { FormControl } from '@angular/forms';
export declare const enum CollectionState {
    Initial = "initial",
    Error = "error",
    Empty = "empty",
    Data = "data"
}
export declare class Collection<T> {
    private readonly _provider;
    private readonly _initialValues;
    readonly queryControl: FormControl;
    readonly queryFieldsControl: FormControl;
    readonly pageNumberControl: FormControl;
    readonly pageSizeControl: FormControl;
    readonly sortControl: FormControl;
    readonly data$: Observable<{
        response?: CollectionData<T>;
        request?: CollectionRequest;
        error?: Error;
        state: CollectionState;
        loading: boolean;
        initial: boolean;
    }>;
    readonly raw$: BehaviorSubject<CollectionData<T>>;
    readonly loading$: BehaviorSubject<boolean>;
    private _formatter;
    private _refresh$;
    private _requestCache;
    constructor(_provider: CollectionProvider<T>, _initialValues?: Partial<CollectionRequest>);
    refresh(): void;
    setFormatter(callback: (items: T) => T): this;
    private getData;
    private formatResponse;
    private validateRequest;
    private getState;
    private cacheCollectionData;
    private get request();
    private getQueryObservable;
    private controlToObservable;
}
