import * as i0 from '@angular/core';
import { isDevMode, Injectable, NgModule, Optional, SkipSelf, InjectionToken } from '@angular/core';
import { HttpErrorResponse, HTTP_INTERCEPTORS, HttpClientModule } from '@angular/common/http';
import { catchError } from 'rxjs/operators';
import { throwError } from 'rxjs';

class HttpConfig {
    constructor(data) {
        var _a, _b, _c, _d, _e, _f, _g;
        this.parseAliases = (value, standardAlias) => {
            var _a;
            if (typeof value === 'string') {
                value = { [standardAlias]: value };
            }
            if (isDevMode()) {
                const stringRegex = /^[a-zA-Z]+$/;
                for (const alias in value) {
                    if (value.hasOwnProperty(alias) && !stringRegex.test(alias)) {
                        (_a = console === null || console === void 0 ? void 0 : console.warn) === null || _a === void 0 ? void 0 : _a.call(console, `[HttpModule]: Invalid HTTP-alias in HttpConfig named "${alias}"; only latin characters should be used.`);
                    }
                }
            }
            return value;
        };
        // Get the data from the config.
        this.prefix = (_b = (_a = data === null || data === void 0 ? void 0 : data.alias) === null || _a === void 0 ? void 0 : _a.prefix) !== null && _b !== void 0 ? _b : '$';
        this.disableErrorParsing = (_c = data === null || data === void 0 ? void 0 : data.disableErrorParsing) !== null && _c !== void 0 ? _c : false;
        // Parse the aliases.
        const standardAlias = (_e = (_d = data === null || data === void 0 ? void 0 : data.alias) === null || _d === void 0 ? void 0 : _d.standard) !== null && _e !== void 0 ? _e : 'api';
        this.aliases = this.parseAliases(data === null || data === void 0 ? void 0 : data.url, standardAlias);
        // Get the default alias.
        const firstAvailableAlias = (_g = Object.keys((_f = this.aliases) !== null && _f !== void 0 ? _f : {})) === null || _g === void 0 ? void 0 : _g[0];
        this.defaultAlias = `${this.prefix}${firstAvailableAlias}`;
    }
}

class HttpError extends HttpErrorResponse {
    constructor(error) {
        super(error);
    }
    get(token, delimiter = '.') {
        var _a;
        const error = (_a = this.error) !== null && _a !== void 0 ? _a : {};
        return (token !== null && token !== void 0 ? token : '')
            .split(delimiter)
            .reduce((previous, current) => (previous === null || previous === void 0 ? void 0 : previous[current]) ? previous === null || previous === void 0 ? void 0 : previous[current] : null, error);
    }
    has(token, delimiter = '.') {
        return !!this.get(token, delimiter);
    }
    includes(token, value) {
        const array = this.get(token);
        return (array !== null && array !== void 0 ? array : []).includes(value);
    }
}

class BaseUrlInterceptor {
    constructor(_config) {
        var _a, _b;
        this._config = _config;
        // Aliases.
        this._aliases = (_a = this._config) === null || _a === void 0 ? void 0 : _a.aliases;
        this._prefix = (_b = this._config) === null || _b === void 0 ? void 0 : _b.prefix;
        this._extractRegex = new RegExp(`\\${this._prefix}[a-zA-Z]+`);
    }
    intercept(request, next) {
        const url = this.getUrlBasedOnAlias(request.url);
        const modified = request.clone({ url });
        return next.handle(modified);
    }
    getUrlBasedOnAlias(url) {
        const matches = this._extractRegex.exec(url);
        if (!matches) {
            return url;
        }
        return matches.reduce((previous, current) => {
            const alias = this.getAlias(current);
            if (!alias) {
                return previous;
            }
            return previous.replace(current, alias);
        }, url);
    }
    getAlias(value) {
        var _a, _b, _c;
        const alias = value.replace(this._prefix, '');
        const fetched = (_b = (_a = this._aliases) === null || _a === void 0 ? void 0 : _a[alias]) !== null && _b !== void 0 ? _b : null;
        if (fetched) {
            return fetched;
        }
        if (isDevMode()) {
            const available = Object.keys(this._aliases)
                .map(item => `\$${item}`)
                .join(', ');
            (_c = console === null || console === void 0 ? void 0 : console.warn) === null || _c === void 0 ? void 0 : _c.call(console, `[HttpModule]: Used an invalid HTTP-alias named "${value}", skipped parsing; Available aliases is one of (${available}).`);
        }
        return null;
    }
}
BaseUrlInterceptor.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: BaseUrlInterceptor, deps: [{ token: HttpConfig }], target: i0.ɵɵFactoryTarget.Injectable });
BaseUrlInterceptor.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: BaseUrlInterceptor });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: BaseUrlInterceptor, decorators: [{
            type: Injectable
        }], ctorParameters: function () { return [{ type: HttpConfig }]; } });

class ErrorInterceptor {
    constructor(_config) {
        var _a, _b;
        this._config = _config;
        // State.
        this._disabled = (_b = (_a = this._config) === null || _a === void 0 ? void 0 : _a.disableErrorParsing) !== null && _b !== void 0 ? _b : false;
    }
    intercept(request, next) {
        return next.handle(request).pipe(catchError(error => {
            // If we have a matching http error response
            // re-map it with our custom class (if not disabled).
            if (!this._disabled && error instanceof HttpErrorResponse) {
                const parsed = new HttpError(error);
                return throwError(parsed);
            }
            // We did not match anything, just re-trow the error.
            return throwError(error);
        }));
    }
}
ErrorInterceptor.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: ErrorInterceptor, deps: [{ token: HttpConfig }], target: i0.ɵɵFactoryTarget.Injectable });
ErrorInterceptor.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: ErrorInterceptor });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: ErrorInterceptor, decorators: [{
            type: Injectable
        }], ctorParameters: function () { return [{ type: HttpConfig }]; } });

class HttpModule {
    constructor(_parent) {
        this._parent = _parent;
        if (this._parent) {
            throw new Error('HttpModule is already loaded; Import it in the core module only.');
        }
    }
    static forRoot(config) {
        return {
            ngModule: HttpModule,
            providers: [
                { provide: HTTP_CONFIGURATION_DATA, useValue: config },
                { provide: HttpConfig, useFactory: createHttpConfig, deps: [HTTP_CONFIGURATION_DATA] },
                { provide: HTTP_INTERCEPTORS, useClass: BaseUrlInterceptor, multi: true },
                { provide: HTTP_INTERCEPTORS, useClass: ErrorInterceptor, multi: true }
            ]
        };
    }
}
HttpModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: HttpModule, deps: [{ token: HttpModule, optional: true, skipSelf: true }], target: i0.ɵɵFactoryTarget.NgModule });
HttpModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: HttpModule, imports: [HttpClientModule] });
HttpModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: HttpModule, imports: [[HttpClientModule]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: HttpModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [HttpClientModule]
                }]
        }], ctorParameters: function () {
        return [{ type: HttpModule, decorators: [{
                        type: Optional
                    }, {
                        type: SkipSelf
                    }] }];
    } });
const HTTP_CONFIGURATION_DATA = new InjectionToken('HTTP_CONFIGURATION_DATA');
function createHttpConfig(data) {
    return new HttpConfig(data);
}

/**
 * Generated bundle index. Do not edit.
 */

export { BaseUrlInterceptor, ErrorInterceptor, HTTP_CONFIGURATION_DATA, HttpConfig, HttpError, HttpModule, createHttpConfig };
